package bug;

import java.sql.Connection;
import java.sql.PreparedStatement;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.naming.InitialContext;
import javax.sql.DataSource;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import javax.transaction.UserTransaction;

@Singleton
@Startup
@TransactionManagement(value=TransactionManagementType.BEAN)
public class TransactionHandlingBug {

	@PostConstruct
	public void start() throws Exception {
		getUserTransaction().begin();
		try {
			Transaction suspendedTx = getTransactionManager().suspend();

			{// parallel tx
				getUserTransaction().begin();
				try {
					executeStatement();
					getUserTransaction().commit();
				} catch (Exception e) {
					e.printStackTrace();
					getUserTransaction().rollback();
				}
			}

			getTransactionManager().resume(suspendedTx);
			executeStatement();
			getUserTransaction().commit();
		} catch (Exception e) {
			e.printStackTrace();
			getUserTransaction().rollback();
		}

		for (int i = 0; i < 20; i++) {
			getUserTransaction().begin();
			try {
				executeStatement();
				getUserTransaction().commit();
			} catch (Exception e) {
				e.printStackTrace();
				getUserTransaction().rollback();
			}
		}
	}

	private TransactionManager getTransactionManager() throws Exception {
		return (TransactionManager) new InitialContext().lookup("java:appserver/TransactionManager");
	}

	private UserTransaction getUserTransaction() throws Exception {
		return (UserTransaction) new InitialContext().lookup("java:comp/UserTransaction");
	}

	private DataSource getDataSource() throws Exception {
		return (DataSource) new InitialContext().lookup("jdbc/__TimerPool");
	}

	private Connection getConnection() throws Exception {
		Connection connection = getDataSource().getConnection();
		connection.setAutoCommit(false);
		return connection;
	}

	private boolean executeStatement() throws Exception {
		PreparedStatement stmt = getConnection().prepareStatement("SELECT 1 FROM DUAL");
		return stmt.execute();
	}
}
